/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef IDIOSKOPOSFACTORY_H
#define IDIOSKOPOSFACTORY_H

#include <libxml++/libxml++.h>
#include <glibmm.h>
#include <idioskopos/object.h>

namespace Idioskopos
{
  class Object;

  /** Initialize the XML property factory */
  void xml_property_factory_init();

  /** Get the DTD of the Idioskopos XML format */
  const Glib::ustring& xml_dtd(bool as_inline=false, bool with_header=false);

  /** Register a property type with the factory */
  void register_property_factory_type( const Glib::ustring& class_name,
                                       const Glib::ustring& value_type,
                                       sigc::slot<Object::pointer> slot );

  /** Unregister (remove) a property type from the factory */
  void unregister_property_factory_type( const Glib::ustring& class_name,
                                         const Glib::ustring& value_type );

  /** Register an introspectable type with the factory */
  void register_introspectable_factory_type( const Glib::ustring& class_name,
                                             sigc::slot<Object::pointer> slot );

  /** Unregister (remove) an introspectable type from the factory */
  void unregister_introspectable_factory_type( const Glib::ustring& class_name );

  /**
   * Given a string XML specification of an Idioskopos object,
   * dynamically creates the object on the stack if the factory has the
   * appropriate types available.
   */
  Object::pointer xml_factory_create( const Glib::ustring& xml_initialization );

  /**
   * Given a libxml++ node of an Idioskopos object, dynamically
   * creates the object on the stack if the factory has the appropriate
   * types available.
   */
  Object::pointer xml_factory_create( const xmlpp::Node* n );

  /**
   * Given a libxml++ node of an Idioskopos property, dynamically
   * creates the object on the stack if the factory has the appropriate
   * types available.
   */
  Object::pointer xml_property_factory_create( const xmlpp::Node* n );

  /**
   * Given a libxml++ node of an Idioskopos introspectable object,
   * dynamically creates the object on the stack if the factory has
   * the appropriate types available.
   */
  Object::pointer xml_introspectable_factory_create( const xmlpp::Node* n );

}

/**
 * Convenience macro for creating the base methods for an introspectable
 * object, and registering it with the factory.
 */
#define IDIOSKOPOS_OBJECT_AND_REGISTRAR(idioskopos_name)                                     \
  virtual const Glib::ustring& class_name() const {                                          \
    static Glib::ustring s = (#idioskopos_name);                                             \
    return (s);                                                                              \
  }                                                                                          \
                                                                                             \
  static Idioskopos::Object::pointer create_object() {                                       \
    return Idioskopos::Object::pointer( new idioskopos_name(Glib::ustring()) );              \
  }                                                                                          \
                                                                                             \
  struct idioskopos_registrar {                                                              \
    idioskopos_registrar() {                                                                 \
      static bool initialized = false;                                                       \
      if ( ! initialized )                                                                   \
        register_introspectable_factory_type(#idioskopos_name,                               \
                                             sigc::ptr_fun(idioskopos_name::create_object)); \
      initialized = true;                                                                    \
    }                                                                                        \
  };                                                                                         \
                                                                                             \
  static idioskopos_registrar idioskopos_name ## _idioskopos_registrar_static

/**
 * Convenience macro for creating the base methods for an introspectable
 * object, and registering it with the factory with a name different from
 * the C++ class name.
 */
#define IDIOSKOPOS_OBJECT_NAME_AND_REGISTRAR(idioskopos_name, idioskopos_strname)            \
  virtual const Glib::ustring& class_name() const {                                          \
    static Glib::ustring s = (#idioskopos_strname);                                          \
    return (s);                                                                              \
  }                                                                                          \
                                                                                             \
  static Idioskopos::Object::pointer create_object() {                                       \
    return Idioskopos::Object::pointer( new idioskopos_name(Glib::ustring()) );              \
  }                                                                                          \
                                                                                             \
  struct idioskopos_registrar {                                                              \
    idioskopos_registrar() {                                                                 \
      static bool initialized = false;                                                       \
      if ( ! initialized )                                                                   \
        register_introspectable_factory_type(#idioskopos_strname,                            \
                                             sigc::ptr_fun(idioskopos_name::create_object)); \
      initialized = true;                                                                    \
    }                                                                                        \
  };                                                                                         \
                                                                                             \
  static idioskopos_registrar idioskopos_name ## _idioskopos_registrar_static

/**
 * Convenience macro for creating the base methods for an introspectable
 * object, and registering it with the factory with a name different from
 * the C++ class name and provides a slot to a custom create_object() method.
 */
#define IDIOSKOPOS_OBJECT_NAME_SLOT_AND_REGISTRAR(idioskopos_name, idioskopos_strname, idioskopos_create_slot) \
  virtual const Glib::ustring& class_name() const {                                                            \
    static Glib::ustring s = (#idioskopos_strname);                                                            \
    return (s);                                                                                                \
  }                                                                                                            \
                                                                                                               \
  struct idioskopos_registrar {                                                                                \
    idioskopos_registrar() {                                                                                   \
      static bool initialized = false;                                                                         \
      if ( ! initialized )                                                                                     \
        register_introspectable_factory_type(#idioskopos_strname, idioskopos_create_slot);                     \
      initialized = true;                                                                                      \
}                                                                                                              \
};                                                                                                             \
                                                                                                               \
  static idioskopos_registrar idioskopos_name ## _idioskopos_registrar_static

#define IDIOSKOPOS_REGISTRAR(idioskopos_name)                                             \
  idioskopos_name::idioskopos_registrar idioskopos_name ## _idioskopos_registrar_static


#endif
